@@ -13,6 +13,7 @@
 #include "include/v8-proxy.h"
 #include "include/v8-typed-array.h"
 #include "include/v8-wasm.h"
+#include "src/base/contextual.h"
 #include "src/execution/isolate.h"
 #include "src/objects/bigint.h"
 #include "src/objects/contexts.h"
@@ -29,6 +30,7 @@
 
 namespace v8 {
 
+class DictionaryTemplate;
 class Extension;
 class Signature;
 class Template;
@@ -46,29 +48,23 @@ class Script;
 class EphemeronTable;
 }  // namespace debug
 
-// Constants used in the implementation of the API.  The most natural thing
-// would usually be to place these with the classes that use them, but
-// we want to keep them out of v8.h because it is an externally
-// visible file.
-class Consts {
- public:
-  enum TemplateType { FUNCTION_TEMPLATE = 0, OBJECT_TEMPLATE = 1 };
-};
-
-template <typename T>
-inline T ToCData(v8::internal::Tagged<v8::internal::Object> obj);
-
-template <>
+template <typename T, internal::ExternalPointerTag tag>
+inline T ToCData(i::Isolate* isolate,
+                 v8::internal::Tagged<v8::internal::Object> obj);
+template <internal::ExternalPointerTag tag>
 inline v8::internal::Address ToCData(
+    v8::internal::Isolate* isolate,
     v8::internal::Tagged<v8::internal::Object> obj);
 
-template <typename T>
-inline v8::internal::Handle<v8::internal::Object> FromCData(
-    v8::internal::Isolate* isolate, T obj);
+template <internal::ExternalPointerTag tag, typename T>
+inline v8::internal::Handle<
+    v8::internal::UnionOf<v8::internal::Smi, v8::internal::Foreign>>
+FromCData(v8::internal::Isolate* isolate, T obj);
 
-template <>
-inline v8::internal::Handle<v8::internal::Object> FromCData(
-    v8::internal::Isolate* isolate, v8::internal::Address obj);
+template <internal::ExternalPointerTag tag>
+inline v8::internal::Handle<
+    v8::internal::UnionOf<v8::internal::Smi, v8::internal::Foreign>>
+FromCData(v8::internal::Isolate* isolate, v8::internal::Address obj);
 
 class ApiFunction {
  public:
@@ -95,124 +91,151 @@ class RegisteredExtension {
   static RegisteredExtension* first_extension_;
 };
 
-#define TO_LOCAL_LIST(V)                               \
-  V(ToLocal, AccessorPair, debug::AccessorPair)        \
-  V(ToLocal, NativeContext, Context)                   \
-  V(ToLocal, Object, Value)                            \
-  V(ToLocal, Module, Module)                           \
-  V(ToLocal, Name, Name)                               \
-  V(ToLocal, String, String)                           \
-  V(ToLocal, Symbol, Symbol)                           \
-  V(ToLocal, JSRegExp, RegExp)                         \
-  V(ToLocal, JSReceiver, Object)                       \
-  V(ToLocal, JSObject, Object)                         \
-  V(ToLocal, JSFunction, Function)                     \
-  V(ToLocal, JSArray, Array)                           \
-  V(ToLocal, JSMap, Map)                               \
-  V(ToLocal, JSSet, Set)                               \
-  V(ToLocal, JSProxy, Proxy)                           \
-  V(ToLocal, JSArrayBuffer, ArrayBuffer)               \
-  V(ToLocal, JSArrayBufferView, ArrayBufferView)       \
-  V(ToLocal, JSDataView, DataView)                     \
-  V(ToLocal, JSRabGsabDataView, DataView)              \
-  V(ToLocal, JSTypedArray, TypedArray)                 \
-  V(ToLocalShared, JSArrayBuffer, SharedArrayBuffer)   \
-  V(ToLocal, FunctionTemplateInfo, FunctionTemplate)   \
-  V(ToLocal, ObjectTemplateInfo, ObjectTemplate)       \
-  V(SignatureToLocal, FunctionTemplateInfo, Signature) \
-  V(MessageToLocal, Object, Message)                   \
-  V(PromiseToLocal, JSObject, Promise)                 \
-  V(StackTraceToLocal, FixedArray, StackTrace)         \
-  V(StackFrameToLocal, StackFrameInfo, StackFrame)     \
-  V(NumberToLocal, Object, Number)                     \
-  V(IntegerToLocal, Object, Integer)                   \
-  V(Uint32ToLocal, Object, Uint32)                     \
-  V(ToLocal, BigInt, BigInt)                           \
-  V(ExternalToLocal, JSObject, External)               \
-  V(CallableToLocal, JSReceiver, Function)             \
-  V(ToLocalPrimitive, Object, Primitive)               \
-  V(FixedArrayToLocal, FixedArray, FixedArray)         \
-  V(PrimitiveArrayToLocal, FixedArray, PrimitiveArray) \
-  V(ToLocal, ScriptOrModule, ScriptOrModule)
+#define TO_LOCAL_LIST(V)                                 \
+  V(ToLocal, AccessorPair, debug::AccessorPair)          \
+  V(ToLocal, NativeContext, Context)                     \
+  V(ToLocal, Object, Value)                              \
+  V(ToLocal, Module, Module)                             \
+  V(ToLocal, Name, Name)                                 \
+  V(ToLocal, String, String)                             \
+  V(ToLocal, Symbol, Symbol)                             \
+  V(ToLocal, JSRegExp, RegExp)                           \
+  V(ToLocal, JSReceiver, Object)                         \
+  V(ToLocal, JSObject, Object)                           \
+  V(ToLocal, JSFunction, Function)                       \
+  V(ToLocal, JSArray, Array)                             \
+  V(ToLocal, JSMap, Map)                                 \
+  V(ToLocal, JSSet, Set)                                 \
+  V(ToLocal, JSProxy, Proxy)                             \
+  V(ToLocal, JSArrayBuffer, ArrayBuffer)                 \
+  V(ToLocal, JSArrayBufferView, ArrayBufferView)         \
+  V(ToLocal, JSDataView, DataView)                       \
+  V(ToLocal, JSRabGsabDataView, DataView)                \
+  V(ToLocal, JSTypedArray, TypedArray)                   \
+  V(ToLocalShared, JSArrayBuffer, SharedArrayBuffer)     \
+  V(ToLocal, FunctionTemplateInfo, FunctionTemplate)     \
+  V(ToLocal, ObjectTemplateInfo, ObjectTemplate)         \
+  V(ToLocal, DictionaryTemplateInfo, DictionaryTemplate) \
+  V(SignatureToLocal, FunctionTemplateInfo, Signature)   \
+  V(MessageToLocal, Object, Message)                     \
+  V(PromiseToLocal, JSObject, Promise)                   \
+  V(StackTraceToLocal, FixedArray, StackTrace)           \
+  V(StackFrameToLocal, StackFrameInfo, StackFrame)       \
+  V(NumberToLocal, Object, Number)                       \
+  V(IntegerToLocal, Object, Integer)                     \
+  V(Uint32ToLocal, Object, Uint32)                       \
+  V(ToLocal, BigInt, BigInt)                             \
+  V(ExternalToLocal, JSObject, External)                 \
+  V(CallableToLocal, JSReceiver, Function)               \
+  V(ToLocalPrimitive, Object, Primitive)                 \
+  V(FixedArrayToLocal, FixedArray, FixedArray)           \
+  V(PrimitiveArrayToLocal, FixedArray, PrimitiveArray)   \
+  V(ToLocal, ScriptOrModule, ScriptOrModule)             \
+  IF_WASM(V, ToLocal, WasmModuleObject, WasmModuleObject)
 
-#define OPEN_HANDLE_LIST(V)                    \
-  V(Template, TemplateInfo)                    \
-  V(FunctionTemplate, FunctionTemplateInfo)    \
-  V(ObjectTemplate, ObjectTemplateInfo)        \
-  V(Signature, FunctionTemplateInfo)           \
-  V(Data, Object)                              \
-  V(RegExp, JSRegExp)                          \
-  V(Object, JSReceiver)                        \
-  V(Array, JSArray)                            \
-  V(Map, JSMap)                                \
-  V(Set, JSSet)                                \
-  V(ArrayBuffer, JSArrayBuffer)                \
-  V(ArrayBufferView, JSArrayBufferView)        \
-  V(TypedArray, JSTypedArray)                  \
-  V(Uint8Array, JSTypedArray)                  \
-  V(Uint8ClampedArray, JSTypedArray)           \
-  V(Int8Array, JSTypedArray)                   \
-  V(Uint16Array, JSTypedArray)                 \
-  V(Int16Array, JSTypedArray)                  \
-  V(Uint32Array, JSTypedArray)                 \
-  V(Int32Array, JSTypedArray)                  \
-  V(Float32Array, JSTypedArray)                \
-  V(Float64Array, JSTypedArray)                \
-  V(DataView, JSDataViewOrRabGsabDataView)     \
-  V(SharedArrayBuffer, JSArrayBuffer)          \
-  V(Name, Name)                                \
-  V(String, String)                            \
-  V(Symbol, Symbol)                            \
-  V(Script, JSFunction)                        \
-  V(UnboundModuleScript, SharedFunctionInfo)   \
-  V(UnboundScript, SharedFunctionInfo)         \
-  V(Module, Module)                            \
-  V(Function, JSReceiver)                      \
-  V(Message, JSMessageObject)                  \
-  V(Context, NativeContext)                    \
-  V(External, Object)                          \
-  V(StackTrace, FixedArray)                    \
-  V(StackFrame, StackFrameInfo)                \
-  V(Proxy, JSProxy)                            \
-  V(debug::GeneratorObject, JSGeneratorObject) \
-  V(debug::ScriptSource, HeapObject)           \
-  V(debug::Script, Script)                     \
-  V(debug::EphemeronTable, EphemeronHashTable) \
-  V(debug::AccessorPair, AccessorPair)         \
-  V(Promise, JSPromise)                        \
-  V(Primitive, Object)                         \
-  V(PrimitiveArray, FixedArray)                \
-  V(BigInt, BigInt)                            \
-  V(ScriptOrModule, ScriptOrModule)            \
-  V(FixedArray, FixedArray)                    \
-  V(ModuleRequest, ModuleRequest)              \
+#define TO_LOCAL_NAME_LIST(V) \
+  V(ToLocal)                  \
+  V(ToLocalShared)            \
+  V(SignatureToLocal)         \
+  V(MessageToLocal)           \
+  V(PromiseToLocal)           \
+  V(StackTraceToLocal)        \
+  V(StackFrameToLocal)        \
+  V(NumberToLocal)            \
+  V(IntegerToLocal)           \
+  V(Uint32ToLocal)            \
+  V(ExternalToLocal)          \
+  V(CallableToLocal)          \
+  V(ToLocalPrimitive)         \
+  V(FixedArrayToLocal)        \
+  V(PrimitiveArrayToLocal)
+
+#define OPEN_HANDLE_LIST(V)                     \
+  V(Template, TemplateInfo)                     \
+  V(FunctionTemplate, FunctionTemplateInfo)     \
+  V(ObjectTemplate, ObjectTemplateInfo)         \
+  V(DictionaryTemplate, DictionaryTemplateInfo) \
+  V(Signature, FunctionTemplateInfo)            \
+  V(Data, Object)                               \
+  V(Number, Number)                             \
+  V(RegExp, JSRegExp)                           \
+  V(Object, JSReceiver)                         \
+  V(Array, JSArray)                             \
+  V(Map, JSMap)                                 \
+  V(Set, JSSet)                                 \
+  V(ArrayBuffer, JSArrayBuffer)                 \
+  V(ArrayBufferView, JSArrayBufferView)         \
+  V(TypedArray, JSTypedArray)                   \
+  V(Uint8Array, JSTypedArray)                   \
+  V(Uint8ClampedArray, JSTypedArray)            \
+  V(Int8Array, JSTypedArray)                    \
+  V(Uint16Array, JSTypedArray)                  \
+  V(Int16Array, JSTypedArray)                   \
+  V(Uint32Array, JSTypedArray)                  \
+  V(Int32Array, JSTypedArray)                   \
+  V(Float16Array, JSTypedArray)                 \
+  V(Float32Array, JSTypedArray)                 \
+  V(Float64Array, JSTypedArray)                 \
+  V(DataView, JSDataViewOrRabGsabDataView)      \
+  V(SharedArrayBuffer, JSArrayBuffer)           \
+  V(Name, Name)                                 \
+  V(String, String)                             \
+  V(Symbol, Symbol)                             \
+  V(Script, JSFunction)                         \
+  V(UnboundModuleScript, SharedFunctionInfo)    \
+  V(UnboundScript, SharedFunctionInfo)          \
+  V(Module, Module)                             \
+  V(Function, JSReceiver)                       \
+  V(CompileHintsCollector, Script)              \
+  V(Message, JSMessageObject)                   \
+  V(Context, NativeContext)                     \
+  V(External, Object)                           \
+  V(StackTrace, FixedArray)                     \
+  V(StackFrame, StackFrameInfo)                 \
+  V(Proxy, JSProxy)                             \
+  V(debug::GeneratorObject, JSGeneratorObject)  \
+  V(debug::ScriptSource, HeapObject)            \
+  V(debug::Script, Script)                      \
+  V(debug::EphemeronTable, EphemeronHashTable)  \
+  V(debug::AccessorPair, AccessorPair)          \
+  V(Promise, JSPromise)                         \
+  V(Primitive, Object)                          \
+  V(PrimitiveArray, FixedArray)                 \
+  V(BigInt, BigInt)                             \
+  V(ScriptOrModule, ScriptOrModule)             \
+  V(FixedArray, FixedArray)                     \
+  V(ModuleRequest, ModuleRequest)               \
   IF_WASM(V, WasmMemoryObject, WasmMemoryObject)
 
 class Utils {
  public:
-  static inline bool ApiCheck(bool condition, const char* location,
-                              const char* message) {
-    if (!condition) Utils::ReportApiFailure(location, message);
+  static V8_INLINE bool ApiCheck(bool condition, const char* location,
+                                 const char* message) {
+    if (V8_UNLIKELY(!condition)) {
+      Utils::ReportApiFailure(location, message);
+    }
     return condition;
   }
   static void ReportOOMFailure(v8::internal::Isolate* isolate,
                                const char* location, const OOMDetails& details);
 
-#define DECLARE_TO_LOCAL(Name, From, To)                  \
-  static inline Local<v8::To> Name(                       \
-      v8::internal::Handle<v8::internal::From> obj);      \
-  static inline Local<v8::To> Name(                       \
-      v8::internal::DirectHandle<v8::internal::From> obj, \
-      v8::internal::Isolate* isolate);
+  // TODO(42203211): It would be nice if we could keep only a version with
+  // direct handles. But the implicit conversion from handles to direct handles
+  // combined with the heterogeneous copy constructor for direct handles make
+  // this ambiguous.
+  // TODO(42203211): Use C++20 concepts instead of the enable_if trait, when
+  // they are fully supported in V8.
+#define DECLARE_TO_LOCAL(Name)                                     \
+  template <template <typename T> typename HandleType, typename T, \
+            typename = std::enable_if_t<std::is_convertible_v<     \
+                HandleType<T>, v8::internal::DirectHandle<T>>>>    \
+  static inline auto Name(HandleType<T> obj);
 
-  TO_LOCAL_LIST(DECLARE_TO_LOCAL)
+  TO_LOCAL_NAME_LIST(DECLARE_TO_LOCAL)
 
 #define DECLARE_TO_LOCAL_TYPED_ARRAY(Type, typeName, TYPE, ctype) \
   static inline Local<v8::Type##Array> ToLocal##Type##Array(      \
-      v8::internal::Handle<v8::internal::JSTypedArray> obj);      \
-  static inline Local<v8::Type##Array> ToLocal##Type##Array(      \
-      v8::internal::DirectHandle<v8::internal::JSTypedArray> obj, \
-      v8::internal::Isolate* isolate);
+      v8::internal::DirectHandle<v8::internal::JSTypedArray> obj);
 
   TYPED_ARRAYS(DECLARE_TO_LOCAL_TYPED_ARRAY)
 
@@ -231,11 +254,7 @@ class Utils {
 #undef DECLARE_TO_LOCAL
 
   template <class From, class To>
-  static inline Local<To> Convert(v8::internal::Handle<From> obj);
-
-  template <class From, class To>
-  static inline Local<To> Convert(v8::internal::DirectHandle<From> obj,
-                                  v8::internal::Isolate* isolate);
+  static inline Local<To> Convert(v8::internal::DirectHandle<From> obj);
 
   template <class T>
   static inline v8::internal::Handle<v8::internal::Object> OpenPersistent(
@@ -254,8 +273,22 @@ class Utils {
     return OpenHandle(*handle);
   }
 
+  template <class From, class To>
+  static inline v8::internal::DirectHandle<To> OpenDirectHandle(
+      v8::Local<From> handle) {
+    return OpenDirectHandle(*handle);
+  }
+
  private:
-  static void ReportApiFailure(const char* location, const char* message);
+  V8_NOINLINE V8_PRESERVE_MOST static void ReportApiFailure(
+      const char* location, const char* message);
+
+#define DECLARE_TO_LOCAL_PRIVATE(Name, From, To) \
+  static inline Local<v8::To> Name##_helper(     \
+      v8::internal::DirectHandle<v8::internal::From> obj);
+
+  TO_LOCAL_LIST(DECLARE_TO_LOCAL_PRIVATE)
+#undef DECLARE_TO_LOCAL_PRIVATE
 };
 
 template <class T>
@@ -265,17 +298,10 @@ inline T* ToApi(v8::internal::Handle<v8::internal::Object> obj) {
 
 template <class T>
 inline v8::Local<T> ToApiHandle(
-    v8::internal::Handle<v8::internal::Object> obj) {
+    v8::internal::DirectHandle<v8::internal::Object> obj) {
   return Utils::Convert<v8::internal::Object, T>(obj);
 }
 
-template <class T>
-inline v8::Local<T> ToApiHandle(
-    v8::internal::DirectHandle<v8::internal::Object> obj,
-    v8::internal::Isolate* isolate) {
-  return Utils::Convert<v8::internal::Object, T>(obj, isolate);
-}
-
 template <class T>
 inline bool ToLocal(v8::internal::MaybeHandle<v8::internal::Object> maybe,
                     Local<T>* local) {
@@ -347,12 +373,9 @@ class HandleScopeImplementer {
   inline bool LastEnteredContextWas(Tagged<NativeContext> context);
   inline size_t EnteredContextCount() const { return entered_contexts_.size(); }
 
-  inline void EnterMicrotaskContext(Tagged<NativeContext> context);
-
   // Returns the last entered context or an empty handle if no
   // contexts have been entered.
   inline Handle<NativeContext> LastEnteredContext();
-  inline Handle<NativeContext> LastEnteredOrMicrotaskContext();
 
   inline void SaveContext(Tagged<Context> context);
   inline Tagged<Context> RestoreContext();
@@ -368,13 +391,11 @@ class HandleScopeImplementer {
   }
 
   static const size_t kEnteredContextsOffset;
-  static const size_t kIsMicrotaskContextOffset;
 
  private:
   void ResetAfterArchive() {
     blocks_.detach();
     entered_contexts_.detach();
-    is_microtask_context_.detach();
     saved_contexts_.detach();
     spare_ = nullptr;
     last_handle_before_deferred_block_ = nullptr;
@@ -383,12 +404,10 @@ class HandleScopeImplementer {
   void Free() {
     DCHECK(blocks_.empty());
     DCHECK(entered_contexts_.empty());
-    DCHECK(is_microtask_context_.empty());
     DCHECK(saved_contexts_.empty());
 
     blocks_.free();
     entered_contexts_.free();
-    is_microtask_context_.free();
     saved_contexts_.free();
     if (spare_ != nullptr) {
       DeleteArray(spare_);
@@ -404,15 +423,10 @@ class HandleScopeImplementer {
   DetachableVector<Address*> blocks_;
 
   // Used as a stack to keep track of entered contexts.
-  // If |i|th item of |entered_contexts_| is added by EnterMicrotaskContext,
-  // `is_microtask_context_[i]` is 1.
-  // TODO(tzik): Remove |is_microtask_context_| after the deprecated
-  // v8::Isolate::GetEnteredContext() is removed.
-  DetachableVector<NativeContext> entered_contexts_;
-  DetachableVector<int8_t> is_microtask_context_;
+  DetachableVector<Tagged<NativeContext>> entered_contexts_;
 
   // Used as a stack to keep track of saved contexts.
-  DetachableVector<Context> saved_contexts_;
+  DetachableVector<Tagged<Context>> saved_contexts_;
   Address* spare_;
   Address* last_handle_before_deferred_block_;
   // This is only used for threading support.
@@ -444,10 +458,7 @@ bool HandleScopeImplementer::HasSavedContexts() {
 
 void HandleScopeImplementer::LeaveContext() {
   DCHECK(!entered_contexts_.empty());
-  DCHECK_EQ(entered_contexts_.capacity(), is_microtask_context_.capacity());
-  DCHECK_EQ(entered_contexts_.size(), is_microtask_context_.size());
   entered_contexts_.pop_back();
-  is_microtask_context_.pop_back();
 }
 
 bool HandleScopeImplementer::LastEnteredContextWas(
@@ -472,7 +483,7 @@ void HandleScopeImplementer::DeleteExtensions(internal::Address* prev_limit) {
     // SealHandleScope may make the prev_limit to point inside the block.
     // Cast possibly-unrelated pointers to plain Addres before comparing them
     // to avoid undefined behavior.
-    if (reinterpret_cast<Address>(block_start) <=
+    if (reinterpret_cast<Address>(block_start) <
             reinterpret_cast<Address>(prev_limit) &&
         reinterpret_cast<Address>(prev_limit) <=
             reinterpret_cast<Address>(block_limit)) {
@@ -533,6 +544,8 @@ template <typename T>
 EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE)
 bool ValidateCallbackInfo(const PropertyCallbackInfo<T>& info);
 
+DECLARE_CONTEXTUAL_VARIABLE_WITH_DEFAULT(StackAllocatedCheck, const bool, true);
+
 }  // namespace internal
 }  // namespace v8
 
