@@ -5,6 +5,8 @@
 #ifndef V8_OBJECTS_JS_OBJECTS_INL_H_
 #define V8_OBJECTS_JS_OBJECTS_INL_H_
 
+#include <optional>
+
 #include "src/common/globals.h"
 #include "src/heap/heap-write-barrier.h"
 #include "src/objects/dictionary.h"
@@ -15,6 +17,8 @@
 #include "src/objects/fixed-array.h"
 #include "src/objects/hash-table-inl.h"
 #include "src/objects/heap-number-inl.h"
+#include "src/objects/heap-object-inl.h"
+#include "src/objects/heap-object.h"
 #include "src/objects/instance-type-inl.h"
 #include "src/objects/js-objects.h"
 #include "src/objects/keys.h"
@@ -31,14 +35,14 @@
 // Has to be the last include (doesn't have include guards):
 #include "src/objects/object-macros.h"
 
-namespace v8 {
-namespace internal {
+namespace v8::internal {
 
 #include "torque-generated/src/objects/js-objects-tq-inl.inc"
 
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSReceiver)
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSObject)
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSObjectWithEmbedderSlots)
+TQ_OBJECT_CONSTRUCTORS_IMPL(JSAPIObjectWithEmbedderSlots)
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSCustomElementsObject)
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSSpecialObject)
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSAsyncFromSyncIterator)
@@ -53,8 +57,6 @@ TQ_OBJECT_CONSTRUCTORS_IMPL(JSValidIteratorWrapper)
 
 NEVER_READ_ONLY_SPACE_IMPL(JSReceiver)
 
-CAST_ACCESSOR(JSIteratorResult)
-
 DEF_GETTER(JSObject, elements, Tagged<FixedArrayBase>) {
   return TaggedField<FixedArrayBase, kElementsOffset>::load(cage_base, *this);
 }
@@ -133,7 +135,7 @@ V8_WARN_UNUSED_RESULT MaybeHandle<FixedArray> JSReceiver::OwnPropertyKeys(
 bool JSObject::PrototypeHasNoElements(Isolate* isolate,
                                       Tagged<JSObject> object) {
   DisallowGarbageCollection no_gc;
-  Tagged<HeapObject> prototype = HeapObject::cast(object->map()->prototype());
+  Tagged<HeapObject> prototype = Cast<HeapObject>(object->map()->prototype());
   ReadOnlyRoots roots(isolate);
   Tagged<HeapObject> null = roots.null_value();
   Tagged<FixedArrayBase> empty_fixed_array = roots.empty_fixed_array();
@@ -142,12 +144,12 @@ bool JSObject::PrototypeHasNoElements(Isolate* isolate,
   while (prototype != null) {
     Tagged<Map> map = prototype->map();
     if (IsCustomElementsReceiverMap(map)) return false;
-    Tagged<FixedArrayBase> elements = JSObject::cast(prototype)->elements();
+    Tagged<FixedArrayBase> elements = Cast<JSObject>(prototype)->elements();
     if (elements != empty_fixed_array &&
         elements != empty_slow_element_dictionary) {
       return false;
     }
-    prototype = HeapObject::cast(map->prototype());
+    prototype = Cast<HeapObject>(map->prototype());
   }
   return true;
 }
@@ -223,8 +225,7 @@ void JSObject::EnsureCanContainElements(Handle<JSObject> object,
     if (mode == ALLOW_COPIED_DOUBLE_ELEMENTS) {
       mode = DONT_ALLOW_DOUBLE_ELEMENTS;
     }
-    ObjectSlot objects =
-        Handle<FixedArray>::cast(elements)->GetFirstElementAddress();
+    ObjectSlot objects = Cast<FixedArray>(elements)->RawFieldOfFirstElement();
     EnsureCanContainElements(object, objects, length, mode);
     return;
   }
@@ -233,8 +234,7 @@ void JSObject::EnsureCanContainElements(Handle<JSObject> object,
   if (object->GetElementsKind() == HOLEY_SMI_ELEMENTS) {
     TransitionElementsKind(object, HOLEY_DOUBLE_ELEMENTS);
   } else if (object->GetElementsKind() == PACKED_SMI_ELEMENTS) {
-    Handle<FixedDoubleArray> double_array =
-        Handle<FixedDoubleArray>::cast(elements);
+    auto double_array = Cast<FixedDoubleArray>(elements);
     for (uint32_t i = 0; i < length; ++i) {
       if (double_array->is_the_hole(i)) {
         TransitionElementsKind(object, HOLEY_DOUBLE_ELEMENTS);
@@ -245,8 +245,9 @@ void JSObject::EnsureCanContainElements(Handle<JSObject> object,
   }
 }
 
-void JSObject::SetMapAndElements(Handle<JSObject> object, Handle<Map> new_map,
-                                 Handle<FixedArrayBase> value) {
+void JSObject::SetMapAndElements(DirectHandle<JSObject> object,
+                                 DirectHandle<Map> new_map,
+                                 DirectHandle<FixedArrayBase> value) {
   Isolate* isolate = object->GetIsolate();
   JSObject::MigrateToMap(isolate, object, new_map);
   DCHECK((object->map()->has_fast_smi_or_object_elements() ||
@@ -298,9 +299,10 @@ int JSObject::GetEmbedderFieldsStartOffset() {
 bool JSObject::MayHaveEmbedderFields(Tagged<Map> map) {
   InstanceType instance_type = map->instance_type();
   // TODO(v8) It'd be nice if all objects with embedder data slots inherited
-  // from JSObjectWithEmbedderSlots, but this is currently not possible due to
-  // instance_type constraints.
+  // from JSObjectJSAPIObjectWithEmbedderSlotsWithEmbedderSlots, but this is
+  // currently not possible due to instance_type constraints.
   return InstanceTypeChecker::IsJSObjectWithEmbedderSlots(instance_type) ||
+         InstanceTypeChecker::IsJSAPIObjectWithEmbedderSlots(instance_type) ||
          InstanceTypeChecker::IsJSSpecialObject(instance_type);
 }
 
@@ -345,24 +347,29 @@ void JSObject::SetEmbedderField(int index, Tagged<Smi> value) {
   EmbedderDataSlot(Tagged(*this), index).store_smi(value);
 }
 
-bool JSObject::IsDroppableApiObject() const {
-  auto instance_type = map()->instance_type();
+// static
+bool JSObject::IsDroppableApiObject(const Tagged<Map> map) {
+  auto instance_type = map->instance_type();
   return InstanceTypeChecker::IsJSApiObject(instance_type) ||
          instance_type == JS_SPECIAL_API_OBJECT_TYPE;
 }
 
+bool JSObject::IsDroppableApiObject() const {
+  return IsDroppableApiObject(map());
+}
+
 // Access fast-case object properties at index. The use of these routines
 // is needed to correctly distinguish between properties stored in-object and
 // properties stored in the properties array.
-Tagged<Object> JSObject::RawFastPropertyAt(FieldIndex index) const {
+Tagged<JSAny> JSObject::RawFastPropertyAt(FieldIndex index) const {
   PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
   return RawFastPropertyAt(cage_base, index);
 }
 
-Tagged<Object> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
-                                           FieldIndex index) const {
+Tagged<JSAny> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
+                                          FieldIndex index) const {
   if (index.is_inobject()) {
-    return TaggedField<Object>::Relaxed_Load(cage_base, *this, index.offset());
+    return TaggedField<JSAny>::Relaxed_Load(cage_base, *this, index.offset());
   } else {
     return property_array(cage_base)->get(cage_base,
                                           index.outobject_array_index());
@@ -371,24 +378,24 @@ Tagged<Object> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
 
 // The SeqCst versions of RawFastPropertyAt are used for atomically accessing
 // shared struct fields.
-Tagged<Object> JSObject::RawFastPropertyAt(FieldIndex index,
-                                           SeqCstAccessTag tag) const {
+Tagged<JSAny> JSObject::RawFastPropertyAt(FieldIndex index,
+                                          SeqCstAccessTag tag) const {
   PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
   return RawFastPropertyAt(cage_base, index, tag);
 }
 
-Tagged<Object> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
-                                           FieldIndex index,
-                                           SeqCstAccessTag tag) const {
+Tagged<JSAny> JSObject::RawFastPropertyAt(PtrComprCageBase cage_base,
+                                          FieldIndex index,
+                                          SeqCstAccessTag tag) const {
   if (index.is_inobject()) {
-    return TaggedField<Object>::SeqCst_Load(cage_base, *this, index.offset());
+    return TaggedField<JSAny>::SeqCst_Load(cage_base, *this, index.offset());
   } else {
     return property_array(cage_base)->get(cage_base,
                                           index.outobject_array_index(), tag);
   }
 }
 
-base::Optional<Tagged<Object>> JSObject::RawInobjectPropertyAt(
+std::optional<Tagged<Object>> JSObject::RawInobjectPropertyAt(
     PtrComprCageBase cage_base, Tagged<Map> original_map,
     FieldIndex index) const {
   CHECK(index.is_inobject());
@@ -480,10 +487,10 @@ void JSObject::WriteToField(InternalIndex descriptor, PropertyDetails details,
       bits = kHoleNanInt64;
     } else {
       DCHECK(IsHeapNumber(value));
-      bits = HeapNumber::cast(value)->value_as_bits(kRelaxedLoad);
+      bits = Cast<HeapNumber>(value)->value_as_bits();
     }
-    auto box = HeapNumber::cast(RawFastPropertyAt(index));
-    box->set_value_as_bits(bits, kRelaxedStore);
+    auto box = Cast<HeapNumber>(RawFastPropertyAt(index));
+    box->set_value_as_bits(bits);
   } else {
     FastPropertyAtPut(index, value);
   }
@@ -594,7 +601,7 @@ void JSObject::InitializeBody(Tagged<Map> map, int start_offset,
     }
     // fill the remainder with one word filler objects (ie just a map word)
     while (offset < size) {
-      Tagged<Object> fm = Object(filler_map.ptr());
+      Tagged<Object> fm = Tagged<Object>(filler_map.ptr());
       WRITE_FIELD(*this, offset, fm);
       offset += kTaggedSize;
     }
@@ -612,9 +619,44 @@ TQ_OBJECT_CONSTRUCTORS_IMPL(JSExternalObject)
 EXTERNAL_POINTER_ACCESSORS(JSExternalObject, value, void*, kValueOffset,
                            kExternalObjectValueTag)
 
-DEF_GETTER(JSGlobalObject, native_context_unchecked, Tagged<Object>) {
-  return TaggedField<Object, kNativeContextOffset>::Relaxed_Load(cage_base,
-                                                                 *this);
+JSApiWrapper::JSApiWrapper(Tagged<JSObject> object) : object_(object) {
+  DCHECK(IsJSApiWrapperObject(object));
+}
+
+template <CppHeapPointerTag lower_bound, CppHeapPointerTag upper_bound>
+void* JSApiWrapper::GetCppHeapWrappable(
+    IsolateForPointerCompression isolate) const {
+  return reinterpret_cast<void*>(
+      object_->ReadCppHeapPointerField<lower_bound, upper_bound>(
+          kCppHeapWrappableOffset, isolate));
+}
+
+void* JSApiWrapper::GetCppHeapWrappable(
+    IsolateForPointerCompression isolate,
+    CppHeapPointerTagRange tag_range) const {
+  return reinterpret_cast<void*>(object_->ReadCppHeapPointerField(
+      kCppHeapWrappableOffset, isolate, tag_range));
+}
+
+template <CppHeapPointerTag tag>
+void JSApiWrapper::SetCppHeapWrappable(IsolateForPointerCompression isolate,
+                                       void* instance) {
+  object_->WriteLazilyInitializedCppHeapPointerField<tag>(
+      JSAPIObjectWithEmbedderSlots::kCppHeapWrappableOffset, isolate,
+      reinterpret_cast<Address>(instance));
+  if (instance) {
+    WriteBarrier::CombinedBarrierForCppHeapPointer(object_, instance);
+  }
+}
+
+void JSApiWrapper::SetCppHeapWrappable(IsolateForPointerCompression isolate,
+                                       void* instance, CppHeapPointerTag tag) {
+  object_->WriteLazilyInitializedCppHeapPointerField(
+      JSAPIObjectWithEmbedderSlots::kCppHeapWrappableOffset, isolate,
+      reinterpret_cast<Address>(instance), tag);
+  if (instance) {
+    WriteBarrier::CombinedBarrierForCppHeapPointer(object_, instance);
+  }
 }
 
 bool JSMessageObject::DidEnsureSourcePositionsAvailable() const {
@@ -623,7 +665,7 @@ bool JSMessageObject::DidEnsureSourcePositionsAvailable() const {
 
 // static
 void JSMessageObject::EnsureSourcePositionsAvailable(
-    Isolate* isolate, Handle<JSMessageObject> message) {
+    Isolate* isolate, DirectHandle<JSMessageObject> message) {
   if (message->DidEnsureSourcePositionsAvailable()) {
     DCHECK(message->script()->has_line_ends());
   } else {
@@ -661,7 +703,7 @@ SMI_ACCESSORS(JSMessageObject, raw_type, kMessageTypeOffset)
 DEF_GETTER(JSObject, GetElementsKind, ElementsKind) {
   ElementsKind kind = map(cage_base)->elements_kind();
 #if VERIFY_HEAP && DEBUG
-  Tagged<FixedArrayBase> fixed_array = FixedArrayBase::unchecked_cast(
+  Tagged<FixedArrayBase> fixed_array = UncheckedCast<FixedArrayBase>(
       TaggedField<HeapObject, kElementsOffset>::load(cage_base, *this));
 
   // If a GC was caused while constructing this object, the elements
@@ -769,7 +811,7 @@ DEF_GETTER(JSObject, HasSlowStringWrapperElements, bool) {
 }
 
 DEF_GETTER(JSObject, HasTypedArrayOrRabGsabTypedArrayElements, bool) {
-  DCHECK(!elements(cage_base)->is_null());
+  DCHECK(!elements(cage_base).is_null());
   return map(cage_base)->has_typed_array_or_rab_gsab_typed_array_elements();
 }
 
@@ -798,7 +840,7 @@ RELEASE_ACQUIRE_ACCESSORS_CHECKED2(JSGlobalObject, global_dictionary,
 DEF_GETTER(JSObject, element_dictionary, Tagged<NumberDictionary>) {
   DCHECK(HasDictionaryElements(cage_base) ||
          HasSlowStringWrapperElements(cage_base));
-  return NumberDictionary::cast(elements(cage_base));
+  return Cast<NumberDictionary>(elements(cage_base));
 }
 
 void JSReceiver::initialize_properties(Isolate* isolate) {
@@ -824,8 +866,7 @@ DEF_GETTER(JSReceiver, HasFastProperties, bool) {
       raw_properties_or_hash(cage_base, kRelaxedLoad);
   DCHECK(IsSmi(raw_properties_or_hash_obj) ||
          ((IsGlobalDictionary(raw_properties_or_hash_obj, cage_base) ||
-           IsNameDictionary(raw_properties_or_hash_obj, cage_base) ||
-           IsSwissNameDictionary(raw_properties_or_hash_obj, cage_base)) ==
+           IsPropertyDictionary(raw_properties_or_hash_obj, cage_base)) ==
           map(cage_base)->is_dictionary_map()));
   USE(raw_properties_or_hash_obj);
   return !map(cage_base)->is_dictionary_map();
@@ -840,7 +881,7 @@ DEF_GETTER(JSReceiver, property_dictionary, Tagged<NameDictionary>) {
   if (IsSmi(prop)) {
     return GetReadOnlyRoots(cage_base).empty_property_dictionary();
   }
-  return NameDictionary::cast(prop);
+  return Cast<NameDictionary>(prop);
 }
 
 DEF_GETTER(JSReceiver, property_dictionary_swiss, Tagged<SwissNameDictionary>) {
@@ -852,7 +893,7 @@ DEF_GETTER(JSReceiver, property_dictionary_swiss, Tagged<SwissNameDictionary>) {
   if (IsSmi(prop)) {
     return GetReadOnlyRoots(cage_base).empty_swiss_property_dictionary();
   }
-  return SwissNameDictionary::cast(prop);
+  return Cast<SwissNameDictionary>(prop);
 }
 
 // TODO(gsathya): Pass isolate directly to this function and access
@@ -863,7 +904,24 @@ DEF_GETTER(JSReceiver, property_array, Tagged<PropertyArray>) {
   if (IsSmi(prop) || prop == GetReadOnlyRoots(cage_base).empty_fixed_array()) {
     return GetReadOnlyRoots(cage_base).empty_property_array();
   }
-  return PropertyArray::cast(prop);
+  return Cast<PropertyArray>(prop);
+}
+
+std::optional<Tagged<NativeContext>> JSReceiver::GetCreationContext() {
+  DisallowGarbageCollection no_gc;
+  Tagged<Map> meta_map = map()->map();
+  DCHECK(IsMapMap(meta_map));
+  Tagged<Object> maybe_native_context = meta_map->native_context_or_null();
+  if (IsNull(maybe_native_context)) return {};
+  DCHECK(IsNativeContext(maybe_native_context));
+  return Cast<NativeContext>(maybe_native_context);
+}
+
+MaybeHandle<NativeContext> JSReceiver::GetCreationContext(Isolate* isolate) {
+  DisallowGarbageCollection no_gc;
+  std::optional<Tagged<NativeContext>> maybe_context = GetCreationContext();
+  if (!maybe_context.has_value()) return {};
+  return handle(maybe_context.value(), isolate);
 }
 
 Maybe<bool> JSReceiver::HasProperty(Isolate* isolate, Handle<JSReceiver> object,
@@ -930,6 +988,10 @@ Maybe<PropertyAttributes> JSReceiver::GetOwnElementAttributes(
   return GetPropertyAttributes(&it);
 }
 
+Tagged<NativeContext> JSGlobalObject::native_context() {
+  return *GetCreationContext();
+}
+
 bool JSGlobalObject::IsDetached() {
   return global_proxy()->IsDetachedFrom(*this);
 }
@@ -980,8 +1042,7 @@ static inline bool ShouldConvertToSlowElements(Tagged<JSObject> object,
                                      *new_capacity);
 }
 
-}  // namespace internal
-}  // namespace v8
+}  // namespace v8::internal
 
 #include "src/objects/object-macros-undef.h"
 
