@@ -97,19 +97,36 @@ RUNTIME_FUNCTION(Runtime_TerminateExecution) {
   return isolate->TerminateExecution();
 }
 
-#define THROW_ERROR(isolate, args, call)                               \
-  HandleScope scope(isolate);                                          \
-  DCHECK_LE(1, args.length());                                         \
-  int message_id_smi = args.smi_value_at(0);                           \
-                                                                       \
-  Handle<Object> undefined = isolate->factory()->undefined_value();    \
-  Handle<Object> arg0 = (args.length() > 1) ? args.at(1) : undefined;  \
-  Handle<Object> arg1 = (args.length() > 2) ? args.at(2) : undefined;  \
-  Handle<Object> arg2 = (args.length() > 3) ? args.at(3) : undefined;  \
-                                                                       \
-  MessageTemplate message_id = MessageTemplateFromInt(message_id_smi); \
-                                                                       \
-  THROW_NEW_ERROR_RETURN_FAILURE(isolate, call(message_id, arg0, arg1, arg2));
+namespace {
+
+Tagged<Object> NewError(Isolate* isolate, RuntimeArguments args,
+                        Handle<JSFunction> (Isolate::*constructor_fn)()) {
+  HandleScope scope(isolate);
+  DCHECK_LE(1, args.length());
+  int message_id_smi = args.smi_value_at(0);
+
+  constexpr int kMaxMessageArgs = 3;
+  DirectHandle<Object> message_args[kMaxMessageArgs];
+  int num_message_args = 0;
+  while (num_message_args < kMaxMessageArgs &&
+         args.length() > num_message_args + 1) {
+    message_args[num_message_args] = args.at(num_message_args + 1);
+    ++num_message_args;
+  }
+
+  MessageTemplate message_id = MessageTemplateFromInt(message_id_smi);
+
+  return *isolate->factory()->NewError(
+      (isolate->*constructor_fn)(), message_id,
+      base::VectorOf(message_args, num_message_args));
+}
+
+Tagged<Object> ThrowError(Isolate* isolate, RuntimeArguments args,
+                          Handle<JSFunction> (Isolate::*constructor_fn)()) {
+  return isolate->Throw(NewError(isolate, args, constructor_fn));
+}
+
+}  // namespace
 
 RUNTIME_FUNCTION(Runtime_ThrowRangeError) {
   if (v8_flags.correctness_fuzzer_suppressions) {
@@ -128,22 +145,21 @@ RUNTIME_FUNCTION(Runtime_ThrowRangeError) {
     }
   }
 
-  THROW_ERROR(isolate, args, NewRangeError);
+  return ThrowError(isolate, args, &Isolate::range_error_function);
 }
 
 RUNTIME_FUNCTION(Runtime_ThrowTypeError) {
-  THROW_ERROR(isolate, args, NewTypeError);
+  return ThrowError(isolate, args, &Isolate::type_error_function);
 }
 
 RUNTIME_FUNCTION(Runtime_ThrowTypeErrorIfStrict) {
   if (GetShouldThrow(isolate, Nothing<ShouldThrow>()) ==
-      ShouldThrow::kDontThrow)
+      ShouldThrow::kDontThrow) {
     return ReadOnlyRoots(isolate).undefined_value();
-  THROW_ERROR(isolate, args, NewTypeError);
+  }
+  return ThrowError(isolate, args, &Isolate::type_error_function);
 }
 
-#undef THROW_ERROR
-
 namespace {
 
 const char* ElementsKindToType(ElementsKind fixed_elements_kind) {
@@ -166,7 +182,7 @@ const char* ElementsKindToType(ElementsKind fixed_elements_kind) {
 RUNTIME_FUNCTION(Runtime_ThrowInvalidTypedArrayAlignment) {
   HandleScope scope(isolate);
   DCHECK_EQ(2, args.length());
-  Handle<Map> map = args.at<Map>(0);
+  DirectHandle<Map> map = args.at<Map>(0);
   Handle<String> problem_string = args.at<String>(1);
 
   ElementsKind kind = map->elements_kind();
@@ -191,10 +207,11 @@ RUNTIME_FUNCTION(Runtime_UnwindAndFindExceptionHandler) {
   return isolate->UnwindAndFindHandler();
 }
 
-RUNTIME_FUNCTION(Runtime_PromoteScheduledException) {
+RUNTIME_FUNCTION(Runtime_PropagateException) {
   SealHandleScope shs(isolate);
   DCHECK_EQ(0, args.length());
-  return isolate->PromoteScheduledException();
+  DCHECK(isolate->has_exception());
+  return ReadOnlyRoots(isolate).exception();
 }
 
 RUNTIME_FUNCTION(Runtime_ThrowReferenceError) {
@@ -223,34 +240,8 @@ RUNTIME_FUNCTION(Runtime_NewError) {
   return *isolate->factory()->NewError(message_template, arg0);
 }
 
-RUNTIME_FUNCTION(Runtime_NewForeign) {
-  HandleScope scope(isolate);
-  DCHECK_EQ(0, args.length());
-  return *isolate->factory()->NewForeign(kNullAddress);
-}
-
 RUNTIME_FUNCTION(Runtime_NewTypeError) {
-  HandleScope scope(isolate);
-  DCHECK_LE(args.length(), 4);
-  DCHECK_GE(args.length(), 1);
-  int template_index = args.smi_value_at(0);
-  MessageTemplate message_template = MessageTemplateFromInt(template_index);
-
-  Handle<Object> arg0;
-  if (args.length() >= 2) {
-    arg0 = args.at<Object>(1);
-  }
-
-  Handle<Object> arg1;
-  if (args.length() >= 3) {
-    arg1 = args.at<Object>(2);
-  }
-  Handle<Object> arg2;
-  if (args.length() >= 4) {
-    arg2 = args.at<Object>(3);
-  }
-
-  return *isolate->factory()->NewTypeError(message_template, arg0, arg1, arg2);
+  return NewError(isolate, args, &Isolate::type_error_function);
 }
 
 RUNTIME_FUNCTION(Runtime_NewReferenceError) {
@@ -262,15 +253,6 @@ RUNTIME_FUNCTION(Runtime_NewReferenceError) {
   return *isolate->factory()->NewReferenceError(message_template, arg0);
 }
 
-RUNTIME_FUNCTION(Runtime_NewSyntaxError) {
-  HandleScope scope(isolate);
-  DCHECK_EQ(2, args.length());
-  int template_index = args.smi_value_at(0);
-  Handle<Object> arg0 = args.at(1);
-  MessageTemplate message_template = MessageTemplateFromInt(template_index);
-  return *isolate->factory()->NewSyntaxError(message_template, arg0);
-}
-
 RUNTIME_FUNCTION(Runtime_ThrowInvalidStringLength) {
   HandleScope scope(isolate);
   THROW_NEW_ERROR_RETURN_FAILURE(isolate, NewInvalidStringLengthError());
@@ -306,8 +288,8 @@ RUNTIME_FUNCTION(Runtime_ThrowNoAccess) {
   // TODO(verwaest): We would like to throw using the calling context instead
   // of the entered context but we don't currently have access to that.
   HandleScopeImplementer* impl = isolate->handle_scope_implementer();
-  SaveAndSwitchContext save(
-      isolate, impl->LastEnteredOrMicrotaskContext()->native_context());
+  SaveAndSwitchContext save(isolate,
+                            impl->LastEnteredContext()->native_context());
   THROW_NEW_ERROR_RETURN_FAILURE(isolate,
                                  NewTypeError(MessageTemplate::kNoAccess));
 }
@@ -396,7 +378,7 @@ Tagged<Object> BytecodeBudgetInterruptWithStackCheck(Isolate* isolate,
                                                      CodeKind code_kind) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
-  Handle<JSFunction> function = args.at<JSFunction>(0);
+  DirectHandle<JSFunction> function = args.at<JSFunction>(0);
   TRACE_EVENT0("v8.execute", "V8.BytecodeBudgetInterruptWithStackCheck");
 
   // Check for stack interrupts here so that we can fold the interrupt check
@@ -422,7 +404,7 @@ Tagged<Object> BytecodeBudgetInterrupt(Isolate* isolate, RuntimeArguments& args,
                                        CodeKind code_kind) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
-  Handle<JSFunction> function = args.at<JSFunction>(0);
+  DirectHandle<JSFunction> function = args.at<JSFunction>(0);
   TRACE_EVENT0("v8.execute", "V8.BytecodeBudgetInterrupt");
 
   isolate->tiering_manager()->OnInterruptTick(function, code_kind);
@@ -487,6 +469,7 @@ class SaveAndClearThreadInWasmFlag {};
 
 RUNTIME_FUNCTION(Runtime_AllocateInYoungGeneration) {
   HandleScope scope(isolate);
+  DCHECK(isolate->IsOnCentralStack());
   DCHECK_EQ(2, args.length());
   // TODO(v8:13070): Align allocations in the builtins that call this.
   int size = ALIGN_TO_ALLOCATION_ALIGNMENT(args.smi_value_at(0));
@@ -535,28 +518,6 @@ RUNTIME_FUNCTION(Runtime_AllocateByteArray) {
   return *isolate->factory()->NewByteArray(length);
 }
 
-RUNTIME_FUNCTION(Runtime_AllocateSeqOneByteString) {
-  HandleScope scope(isolate);
-  DCHECK_EQ(1, args.length());
-  int length = args.smi_value_at(0);
-  if (length == 0) return ReadOnlyRoots(isolate).empty_string();
-  Handle<SeqOneByteString> result;
-  ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
-      isolate, result, isolate->factory()->NewRawOneByteString(length));
-  return *result;
-}
-
-RUNTIME_FUNCTION(Runtime_AllocateSeqTwoByteString) {
-  HandleScope scope(isolate);
-  DCHECK_EQ(1, args.length());
-  int length = args.smi_value_at(0);
-  if (length == 0) return ReadOnlyRoots(isolate).empty_string();
-  Handle<SeqTwoByteString> result;
-  ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
-      isolate, result, isolate->factory()->NewRawTwoByteString(length));
-  return *result;
-}
-
 RUNTIME_FUNCTION(Runtime_ThrowIteratorError) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
@@ -638,7 +599,7 @@ RUNTIME_FUNCTION(Runtime_GetAndResetTurboProfilingData) {
 
   std::stringstream stats_stream;
   BasicBlockProfiler::Get()->Log(isolate, stats_stream);
-  Handle<String> result =
+  DirectHandle<String> result =
       isolate->factory()->NewStringFromAsciiChecked(stats_stream.str().c_str());
   BasicBlockProfiler::Get()->ResetCounts(isolate);
   return *result;
@@ -665,7 +626,7 @@ RUNTIME_FUNCTION(Runtime_GetAndResetRuntimeCallStats) {
     // Without arguments, the result is returned as a string.
     std::stringstream stats_stream;
     isolate->counters()->runtime_call_stats()->Print(stats_stream);
-    Handle<String> result = isolate->factory()->NewStringFromAsciiChecked(
+    DirectHandle<String> result = isolate->factory()->NewStringFromAsciiChecked(
         stats_stream.str().c_str());
     isolate->counters()->runtime_call_stats()->Reset();
     return *result;
@@ -674,7 +635,7 @@ RUNTIME_FUNCTION(Runtime_GetAndResetRuntimeCallStats) {
   std::FILE* f;
   if (IsString(args[0])) {
     // With a string argument, the results are appended to that file.
-    Handle<String> filename = args.at<String>(0);
+    DirectHandle<String> filename = args.at<String>(0);
     f = std::fopen(filename->ToCString().get(), "a");
     DCHECK_NOT_NULL(f);
   } else {
@@ -685,7 +646,7 @@ RUNTIME_FUNCTION(Runtime_GetAndResetRuntimeCallStats) {
   }
   // The second argument (if any) is a message header to be printed.
   if (args.length() >= 2) {
-    Handle<String> message = args.at<String>(1);
+    DirectHandle<String> message = args.at<String>(1);
     message->PrintOn(f);
     std::fputc('\n', f);
     std::fflush(f);
@@ -721,14 +682,14 @@ RUNTIME_FUNCTION(Runtime_OrdinaryHasInstance) {
 RUNTIME_FUNCTION(Runtime_Typeof) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
-  Handle<Object> object = args.at(0);
+  DirectHandle<Object> object = args.at(0);
   return *Object::TypeOf(isolate, object);
 }
 
 RUNTIME_FUNCTION(Runtime_AllowDynamicFunction) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
-  Handle<JSFunction> target = args.at<JSFunction>(0);
+  DirectHandle<JSFunction> target = args.at<JSFunction>(0);
   Handle<JSObject> global_proxy(target->global_proxy(), isolate);
   return *isolate->factory()->ToBoolean(
       Builtins::AllowDynamicFunction(isolate, target, global_proxy));
@@ -752,19 +713,19 @@ RUNTIME_FUNCTION(Runtime_CreateAsyncFromSyncIterator) {
                           isolate->factory()->next_string()));
 
   return *isolate->factory()->NewJSAsyncFromSyncIterator(
-      Handle<JSReceiver>::cast(sync_iterator), next);
+      Cast<JSReceiver>(sync_iterator), next);
 }
 
 RUNTIME_FUNCTION(Runtime_GetTemplateObject) {
   HandleScope scope(isolate);
   DCHECK_EQ(3, args.length());
-  Handle<TemplateObjectDescription> description =
+  DirectHandle<TemplateObjectDescription> description =
       args.at<TemplateObjectDescription>(0);
-  Handle<SharedFunctionInfo> shared_info = args.at<SharedFunctionInfo>(1);
+  DirectHandle<SharedFunctionInfo> shared_info = args.at<SharedFunctionInfo>(1);
   int slot_id = args.smi_value_at(2);
 
-  Handle<NativeContext> native_context(isolate->context()->native_context(),
-                                       isolate);
+  DirectHandle<NativeContext> native_context(
+      isolate->context()->native_context(), isolate);
   return *TemplateObjectDescription::GetTemplateObject(
       isolate, native_context, description, shared_info, slot_id);
 }
@@ -778,13 +739,13 @@ RUNTIME_FUNCTION(Runtime_ReportMessageFromMicrotask) {
 
   Handle<Object> exception = args.at(0);
 
-  DCHECK(!isolate->has_pending_exception());
-  isolate->set_pending_exception(*exception);
+  DCHECK(!isolate->has_exception());
+  isolate->set_exception(*exception);
   MessageLocation* no_location = nullptr;
-  Handle<JSMessageObject> message =
+  DirectHandle<JSMessageObject> message =
       isolate->CreateMessageOrAbort(exception, no_location);
   MessageHandler::ReportMessage(isolate, no_location, message);
-  isolate->clear_pending_exception();
+  isolate->clear_exception();
   return ReadOnlyRoots(isolate).undefined_value();
 }
 
@@ -794,7 +755,7 @@ RUNTIME_FUNCTION(Runtime_GetInitializerFunction) {
 
   Handle<JSReceiver> constructor = args.at<JSReceiver>(0);
   Handle<Symbol> key = isolate->factory()->class_fields_symbol();
-  Handle<Object> initializer =
+  DirectHandle<Object> initializer =
       JSReceiver::GetDataProperty(isolate, constructor, key);
   return *initializer;
 }
@@ -807,7 +768,8 @@ RUNTIME_FUNCTION(Runtime_DoubleToStringWithRadix) {
   CHECK(Object::ToInt32(args[1], &radix));
 
   char* const str = DoubleToRadixCString(number, radix);
-  Handle<String> result = isolate->factory()->NewStringFromAsciiChecked(str);
+  DirectHandle<String> result =
+      isolate->factory()->NewStringFromAsciiChecked(str);
   DeleteArray(str);
   return *result;
 }
@@ -822,5 +784,16 @@ RUNTIME_FUNCTION(Runtime_SharedValueBarrierSlow) {
   return *shared_value;
 }
 
+RUNTIME_FUNCTION(Runtime_InvalidateDependentCodeForConstTrackingLet) {
+  HandleScope scope(isolate);
+  DCHECK_EQ(1, args.length());
+  auto const_tracking_let_cell =
+      Cast<ConstTrackingLetCell>(args.at<HeapObject>(0));
+  DependentCode::DeoptimizeDependencyGroups(
+      isolate, *const_tracking_let_cell,
+      DependentCode::kConstTrackingLetChangedGroup);
+  return ReadOnlyRoots(isolate).undefined_value();
+}
+
 }  // namespace internal
 }  // namespace v8
