@@ -16,7 +16,7 @@ const {
 
 const { lstatSync, readdirSync } = require('fs');
 const { lstat, readdir } = require('fs/promises');
-const { join, resolve, basename, isAbsolute } = require('path');
+const { join, resolve, basename, isAbsolute, dirname } = require('path');
 
 const {
   kEmptyObject,
@@ -48,7 +48,7 @@ async function getDirent(path) {
   } catch {
     return null;
   }
-  return new DirentFromStats(basename(path), stat, path);
+  return new DirentFromStats(basename(path), stat, dirname(path));
 }
 
 /**
@@ -60,7 +60,7 @@ function getDirentSync(path) {
   if (stat === undefined) {
     return null;
   }
-  return new DirentFromStats(basename(path), stat, path);
+  return new DirentFromStats(basename(path), stat, dirname(path));
 }
 
 class Cache {
@@ -337,7 +337,7 @@ class Glob {
       const subPatterns = new SafeSet();
       const nSymlinks = new SafeSet();
       for (const index of pattern.indexes) {
-        // For each child, chek potential patterns
+        // For each child, check potential patterns
         if (this.#cache.seen(entryPath, pattern, index) || this.#cache.seen(entryPath, pattern, index + 1)) {
           return;
         }
@@ -365,7 +365,7 @@ class Glob {
             // If next pattern is the last one, add to results
             this.#results.add(entryPath);
           } else if (nextMatches && entry.isDirectory()) {
-            // Pattern mached, meaning two patterns forward
+            // Pattern matched, meaning two patterns forward
             // are also potential patterns
             // e.g **/b/c when entry is a/b - add c to potential patterns
             subPatterns.add(index + 2);
@@ -532,7 +532,7 @@ class Glob {
       const subPatterns = new SafeSet();
       const nSymlinks = new SafeSet();
       for (const index of pattern.indexes) {
-        // For each child, chek potential patterns
+        // For each child, check potential patterns
         if (this.#cache.seen(entryPath, pattern, index) || this.#cache.seen(entryPath, pattern, index + 1)) {
           return;
         }
@@ -566,7 +566,7 @@ class Glob {
               yield this.#withFileTypes ? entry : entryPath;
             }
           } else if (nextMatches && entry.isDirectory()) {
-            // Pattern mached, meaning two patterns forward
+            // Pattern matched, meaning two patterns forward
             // are also potential patterns
             // e.g **/b/c when entry is a/b - add c to potential patterns
             subPatterns.add(index + 2);
@@ -650,7 +650,30 @@ class Glob {
   }
 }
 
+/**
+ * Check if a path matches a glob pattern
+ * @param {string} path the path to check
+ * @param {string} pattern the glob pattern to match
+ * @param {boolean} windows whether the path is on a Windows system, defaults to `isWindows`
+ * @returns {boolean}
+ */
+function matchGlobPattern(path, pattern, windows = isWindows) {
+  validateString(path, 'path');
+  validateString(pattern, 'pattern');
+  return lazyMinimatch().minimatch(path, pattern, {
+    kEmptyObject,
+    nocase: isMacOS || isWindows,
+    windowsPathsNoEscape: true,
+    nonegate: true,
+    nocomment: true,
+    optimizationLevel: 2,
+    platform: windows ? 'win32' : 'posix',
+    nocaseMagicOnly: true,
+  });
+}
+
 module.exports = {
   __proto__: null,
   Glob,
+  matchGlobPattern,
 };
