@@ -49,6 +49,7 @@ const {
   once,
   deprecate,
   isWindows,
+  setOwnProperty,
 } = require('internal/util');
 const { toPathIfFileURL } = require('internal/url');
 const {
@@ -214,6 +215,9 @@ ObjectDefineProperty(Dirent.prototype, 'path', {
   get: deprecate(function() {
     return this.parentPath;
   }, 'dirent.path is deprecated in favor of dirent.parentPath', 'DEP0178'),
+  set(value) {
+    setOwnProperty(this, 'path', value);
+  },
   configurable: true,
   enumerable: false,
 });
@@ -706,38 +710,13 @@ const validateOffsetLengthWrite = hideStackFrames(
   },
 );
 
-/**
- * Validates the given path based on the expected type (file or directory).
- * @param {string} path - The path to validate.
- * @param {string} [propName='path'] - The name of the property being validated.
- * @param {object} options - Additional options for validation.
- * @param {boolean} [options.expectFile] - If true, expects the path to be a file.
- * @param {string} [options.syscall] - The name of the syscall.
- * @throws {TypeError} Throws if the path is not a string, Uint8Array, or URL without null bytes.
- * @throws {Error} Throws if the path does not meet the expected type (file).
- */
-const validatePath = hideStackFrames((path, propName = 'path', options) => {
+const validatePath = hideStackFrames((path, propName = 'path') => {
   if (typeof path !== 'string' && !isUint8Array(path)) {
     throw new ERR_INVALID_ARG_TYPE.HideStackFramesError(propName, ['string', 'Buffer', 'URL'], path);
   }
 
   const pathIsString = typeof path === 'string';
   const pathIsUint8Array = isUint8Array(path);
-  if (options?.expectFile) {
-    const lastCharacter = path[path.length - 1];
-    if (
-      lastCharacter === '/' || lastCharacter === 47 ||
-      (isWindows && (lastCharacter === '\\' || lastCharacter === 92))
-    ) {
-      throw new ERR_FS_EISDIR({
-        code: 'ERR_FS_EISDIR',
-        message: 'is a directory',
-        path,
-        syscall: options.syscall,
-        errno: ERR_FS_EISDIR,
-      });
-    }
-  }
 
   // We can only perform meaningful checks on strings and Uint8Arrays.
   if ((!pathIsString && !pathIsUint8Array) ||
@@ -753,21 +732,11 @@ const validatePath = hideStackFrames((path, propName = 'path', options) => {
   );
 });
 
-/**
- * Validates and returns the given file URL or path based on the expected type (file or directory).
- * @param {string|URL} fileURLOrPath - The file URL or path to validate.
- * @param {string} [propName='path'] - The name of the property being validated.
- * @param {object} options - Additional options for validation.
- * @param {boolean} [options.expectFile] - If true, expects the path to be a file.
- * @param {string} [options.syscall] - The name of the syscall.
- * @returns {string} The validated path.
- */
-const getValidatedPath =
-  hideStackFrames((fileURLOrPath, propName = 'path', options) => {
-    const path = toPathIfFileURL(fileURLOrPath);
-    validatePath(path, propName, options);
-    return path;
-  });
+const getValidatedPath = hideStackFrames((fileURLOrPath, propName = 'path') => {
+  const path = toPathIfFileURL(fileURLOrPath);
+  validatePath(path, propName);
+  return path;
+});
 
 const getValidatedFd = hideStackFrames((fd, propName = 'fd') => {
   if (ObjectIs(fd, -0)) {
