@@ -12,10 +12,7 @@ const {
   NumberParseInt,
   ObjectDefineProperty,
   ObjectFreeze,
-  ObjectGetOwnPropertyDescriptor,
   String,
-  StringPrototypeStartsWith,
-  Symbol,
   globalThis,
 } = primordials;
 
@@ -26,18 +23,14 @@ const {
 } = require('internal/options');
 const { reconnectZeroFillToggle } = require('internal/buffer');
 const {
-  exposeInterface,
   exposeLazyInterfaces,
   defineReplaceableLazyAttribute,
   setupCoverageHooks,
   emitExperimentalWarning,
-  SymbolAsyncDispose,
-  SymbolDispose,
+  deprecate,
 } = require('internal/util');
 
 const {
-  ERR_INVALID_THIS,
-  ERR_NO_CRYPTO,
   ERR_MISSING_OPTION,
   ERR_ACCESS_DENIED,
 } = require('internal/errors').codes;
@@ -47,6 +40,7 @@ const {
     addSerializeCallback,
     isBuildingSnapshot,
   },
+  runDeserializeCallbacks,
 } = require('internal/v8/startup_snapshot');
 
 function prepareMainThreadExecution(expandArgv1 = false, initializeModules = true) {
@@ -106,11 +100,9 @@ function prepareExecution(options) {
   setupNetworkInspection();
   setupNavigator();
   setupWarningHandler();
-  setupUndici();
-  setupWebCrypto();
   setupSQLite();
   setupWebStorage();
-  setupCustomEvent();
+  setupWebsocket();
   setupEventsource();
   setupCodeCoverage();
   setupDebugEnv();
@@ -125,8 +117,6 @@ function prepareExecution(options) {
 
   require('internal/dns/utils').initializeDns();
 
-  setupSymbolDisposePolyfill();
-
   if (isMainThread) {
     assert(internalBinding('worker').isMainThread);
     // Worker threads will get the manifest in the message handler.
@@ -147,7 +137,7 @@ function prepareExecution(options) {
     initializeClusterIPC();
 
     // TODO(joyeecheung): do this for worker threads as well.
-    require('internal/v8/startup_snapshot').runDeserializeCallbacks();
+    runDeserializeCallbacks();
   } else {
     assert(!internalBinding('worker').isMainThread);
     // The setup should be called in LOAD_SCRIPT message handler.
@@ -161,31 +151,6 @@ function prepareExecution(options) {
   return mainEntry;
 }
 
-function setupSymbolDisposePolyfill() {
-  // TODO(MoLow): Remove this polyfill once Symbol.dispose and Symbol.asyncDispose are available in V8.
-  // eslint-disable-next-line node-core/prefer-primordials
-  if (typeof Symbol.dispose !== 'symbol') {
-    ObjectDefineProperty(Symbol, 'dispose', {
-      __proto__: null,
-      configurable: false,
-      enumerable: false,
-      value: SymbolDispose,
-      writable: false,
-    });
-  }
-
-  // eslint-disable-next-line node-core/prefer-primordials
-  if (typeof Symbol.asyncDispose !== 'symbol') {
-    ObjectDefineProperty(Symbol, 'asyncDispose', {
-      __proto__: null,
-      configurable: false,
-      enumerable: false,
-      value: SymbolAsyncDispose,
-      writable: false,
-    });
-  }
-}
-
 function setupUserModules(forceDefaultLoader = false) {
   initializeCJSLoader();
   initializeESMLoader(forceDefaultLoader);
@@ -240,8 +205,7 @@ function patchProcessObject(expandArgv1) {
   let mainEntry;
   // If requested, update process.argv[1] to replace whatever the user provided with the resolved absolute file path of
   // the entry point.
-  if (expandArgv1 && process.argv[1] &&
-      !StringPrototypeStartsWith(process.argv[1], '-')) {
+  if (expandArgv1 && process.argv[1] && process.argv[1][0] !== '-') {
     // Expand process.argv[1] into a full path.
     const path = require('path');
     try {
@@ -310,19 +274,11 @@ function setupWarningHandler() {
   }
 }
 
-// https://fetch.spec.whatwg.org/
 // https://websockets.spec.whatwg.org/
-function setupUndici() {
-  if (getOptionValue('--no-experimental-fetch')) {
-    delete globalThis.fetch;
-    delete globalThis.FormData;
-    delete globalThis.Headers;
-    delete globalThis.Request;
-    delete globalThis.Response;
-  }
-
+function setupWebsocket() {
   if (getOptionValue('--no-experimental-websocket')) {
     delete globalThis.WebSocket;
+    delete globalThis.CloseEvent;
   }
 }
 
@@ -346,43 +302,8 @@ function setupNavigator() {
   defineReplaceableLazyAttribute(globalThis, 'internal/navigator', ['navigator'], false);
 }
 
-// TODO(aduh95): move this to internal/bootstrap/web/* when the CLI flag is
-//               removed.
-function setupWebCrypto() {
-  if (getEmbedderOptions().noBrowserGlobals ||
-      getOptionValue('--no-experimental-global-webcrypto')) {
-    return;
-  }
-
-  if (internalBinding('config').hasOpenSSL) {
-    defineReplaceableLazyAttribute(
-      globalThis,
-      'internal/crypto/webcrypto',
-      ['crypto'],
-      false,
-      function cryptoThisCheck() {
-        if (this !== globalThis && this != null)
-          throw new ERR_INVALID_THIS(
-            'nullish or must be the global object');
-      },
-    );
-    exposeLazyInterfaces(
-      globalThis, 'internal/crypto/webcrypto',
-      ['Crypto', 'CryptoKey', 'SubtleCrypto'],
-    );
-  } else {
-    ObjectDefineProperty(globalThis, 'crypto',
-                         { __proto__: null, ...ObjectGetOwnPropertyDescriptor({
-                           get crypto() {
-                             throw new ERR_NO_CRYPTO();
-                           },
-                         }, 'crypto') });
-
-  }
-}
-
 function setupSQLite() {
-  if (!getOptionValue('--experimental-sqlite')) {
+  if (getOptionValue('--no-experimental-sqlite')) {
     return;
   }
 
@@ -416,17 +337,6 @@ function setupCodeCoverage() {
   }
 }
 
-// TODO(daeyeon): move this to internal/bootstrap/web/* when the CLI flag is
-//                removed.
-function setupCustomEvent() {
-  if (getEmbedderOptions().noBrowserGlobals ||
-      getOptionValue('--no-experimental-global-customevent')) {
-    return;
-  }
-  const { CustomEvent } = require('internal/event_target');
-  exposeInterface(globalThis, 'CustomEvent', CustomEvent);
-}
-
 function setupStacktracePrinterOnSigint() {
   if (!getOptionValue('--trace-sigint')) {
     return;
@@ -524,7 +434,6 @@ function setupNetworkInspection() {
 // this is used to deprecate APIs implemented in C++ where the deprecation
 // utilities are not easily accessible.
 function initializeDeprecations() {
-  const { deprecate } = require('internal/util');
   const pendingDeprecation = getOptionValue('--pending-deprecation');
 
   // DEP0103: access to `process.binding('util').isX` type checkers
@@ -586,8 +495,6 @@ function initializeDeprecations() {
 
 function setupChildProcessIpcChannel() {
   if (process.env.NODE_CHANNEL_FD) {
-    const assert = require('internal/assert');
-
     const fd = NumberParseInt(process.env.NODE_CHANNEL_FD, 10);
     assert(fd >= 0);
 
@@ -613,14 +520,13 @@ function initializeClusterIPC() {
 }
 
 function initializePermission() {
-  const experimentalPermission = getOptionValue('--experimental-permission');
-  if (experimentalPermission) {
+  const permission = getOptionValue('--permission');
+  if (permission) {
     process.binding = function binding(_module) {
       throw new ERR_ACCESS_DENIED('process.binding');
     };
     // Guarantee path module isn't monkey-patched to bypass permission model
     ObjectFreeze(require('path'));
-    emitExperimentalWarning('Permission');
     const { has } = require('internal/process/permission');
     const warnFlags = [
       '--allow-addons',
@@ -672,7 +578,7 @@ function initializePermission() {
     ArrayPrototypeForEach(availablePermissionFlags, (flag) => {
       const value = getOptionValue(flag);
       if (value.length) {
-        throw new ERR_MISSING_OPTION('--experimental-permission');
+        throw new ERR_MISSING_OPTION('--permission');
       }
     });
   }
