@@ -30,6 +30,7 @@ const {
   StringPrototypeTrim,
   Symbol,
   SymbolAsyncIterator,
+  SymbolDispose,
 } = primordials;
 
 const { codes: {
@@ -44,7 +45,7 @@ const {
   validateString,
   validateUint32,
 } = require('internal/validators');
-const { SymbolDispose, kEmptyObject } = require('internal/util');
+const { kEmptyObject } = require('internal/util');
 const {
   inspect,
   getStringWidth,
@@ -261,7 +262,7 @@ function InterfaceConstructor(input, output, completer, terminal) {
 
   function onkeypress(s, key) {
     self[kTtyWrite](s, key);
-    if (key && key.sequence) {
+    if (key?.sequence) {
       // If the key.sequence is half of a surrogate pair
       // (>= 0xd800 and <= 0xdfff), refresh the line so
       // the character is displayed appropriately.
@@ -345,7 +346,7 @@ class Interface extends InterfaceConstructor {
     super(input, output, completer, terminal);
   }
   get columns() {
-    if (this.output && this.output.columns) return this.output.columns;
+    if (this.output?.columns) return this.output.columns;
     return Infinity;
   }
 
@@ -1054,7 +1055,7 @@ class Interface extends InterfaceConstructor {
   // Handle a write from the tty
   [kTtyWrite](s, key) {
     const previousKey = this[kPreviousKey];
-    key = key || kEmptyObject;
+    key ||= kEmptyObject;
     this[kPreviousKey] = key;
 
     if (!key.meta || key.name !== 'y') {
