@@ -17,6 +17,7 @@ const {
   Promise,
   Symbol,
   SymbolAsyncIterator,
+  SymbolDispose,
   globalThis,
 } = primordials;
 
@@ -26,10 +27,6 @@ const {
   validateStringArray,
 } = require('internal/validators');
 
-const {
-  emitExperimentalWarning,
-  SymbolDispose,
-} = require('internal/util');
 const {
   AbortError,
   codes: {
@@ -38,6 +35,8 @@ const {
   },
 } = require('internal/errors');
 
+const { TIMEOUT_MAX } = require('internal/timers');
+
 const PriorityQueue = require('internal/priority_queue');
 const nodeTimers = require('timers');
 const nodeTimersPromises = require('timers/promises');
@@ -62,9 +61,9 @@ function abortIt(signal) {
 }
 
 /**
- * @enum {('setTimeout'|'setInterval'|'setImmediate'|'Date')[]} Supported timers
+ * @enum {('setTimeout'|'setInterval'|'setImmediate'|'Date', 'scheduler.wait')[]} Supported timers
  */
-const SUPPORTED_APIS = ['setTimeout', 'setInterval', 'setImmediate', 'Date'];
+const SUPPORTED_APIS = ['setTimeout', 'setInterval', 'setImmediate', 'Date', 'scheduler.wait'];
 const TIMERS_DEFAULT_INTERVAL = {
   __proto__: null,
   setImmediate: -1,
@@ -106,6 +105,7 @@ class MockTimers {
 
   #realPromisifiedSetTimeout;
   #realPromisifiedSetInterval;
+  #realTimersPromisifiedSchedulerWait;
 
   #realTimersSetTimeout;
   #realTimersClearTimeout;
@@ -130,10 +130,6 @@ class MockTimers {
   #clearInterval = FunctionPrototypeBind(this.#clearTimer, this);
   #clearImmediate = FunctionPrototypeBind(this.#clearTimer, this);
 
-  constructor() {
-    emitExperimentalWarning('The MockTimers API');
-  }
-
   #restoreSetImmediate() {
     ObjectDefineProperty(
       globalThis,
@@ -190,6 +186,13 @@ class MockTimers {
     );
   }
 
+  #restoreOriginalSchedulerWait() {
+    nodeTimersPromises.scheduler.wait = FunctionPrototypeBind(
+      this.#realTimersPromisifiedSchedulerWait,
+      this,
+    );
+  }
+
   #restoreOriginalSetTimeout() {
     ObjectDefineProperty(
       globalThis,
@@ -264,6 +267,14 @@ class MockTimers {
     );
   }
 
+  #storeOriginalSchedulerWait() {
+
+    this.#realTimersPromisifiedSchedulerWait = FunctionPrototypeBind(
+      nodeTimersPromises.scheduler.wait,
+      this,
+    );
+  }
+
   #storeOriginalSetTimeout() {
     this.#realSetTimeout = ObjectGetOwnPropertyDescriptor(
       globalThis,
@@ -288,6 +299,10 @@ class MockTimers {
   }
 
   #createTimer(isInterval, callback, delay, ...args) {
+    if (delay > TIMEOUT_MAX) {
+      delay = 1;
+    }
+
     const timerId = this.#currentTimer++;
     const opts = {
       __proto__: null,
@@ -313,6 +328,9 @@ class MockTimers {
   #createDate() {
     kMock ??= Symbol('MockTimers');
     const NativeDateConstructor = this.#nativeDateDescriptor.value;
+    if (NativeDateConstructor.isMock) {
+      throw new ERR_INVALID_STATE('Date is already being mocked!');
+    }
     /**
      * Function to mock the Date constructor, treats cases as per ECMA-262
      * and returns a Date object with a mocked implementation
@@ -377,7 +395,7 @@ class MockTimers {
       return FunctionPrototypeToString(MockDate[kMock].#nativeDateDescriptor.value);
     };
 
-    // We need to polute the prototype of this
+    // We need to pollute the prototype of this
     ObjectDefineProperties(MockDate, {
       __proto__: null,
       [kMock]: {
@@ -556,8 +574,14 @@ class MockTimers {
     const options = {
       __proto__: null,
       toFake: {
-        __proto__: null,
-        setTimeout: () => {
+        '__proto__': null,
+        'scheduler.wait': () => {
+          this.#storeOriginalSchedulerWait();
+
+          nodeTimersPromises.scheduler.wait = (delay, options) =>
+            this.#setTimeoutPromisified(delay, undefined, options);
+        },
+        'setTimeout': () => {
           this.#storeOriginalSetTimeout();
 
           globalThis.setTimeout = this.#setTimeout;
@@ -571,7 +595,7 @@ class MockTimers {
             this,
           );
         },
-        setInterval: () => {
+        'setInterval': () => {
           this.#storeOriginalSetInterval();
 
           globalThis.setInterval = this.#setInterval;
@@ -585,7 +609,7 @@ class MockTimers {
             this,
           );
         },
-        setImmediate: () => {
+        'setImmediate': () => {
           this.#storeOriginalSetImmediate();
 
           // setImmediate functions needs to bind MockTimers
@@ -609,23 +633,26 @@ class MockTimers {
             this,
           );
         },
-        Date: () => {
+        'Date': () => {
           this.#nativeDateDescriptor = ObjectGetOwnPropertyDescriptor(globalThis, 'Date');
           globalThis.Date = this.#createDate();
         },
       },
       toReal: {
-        __proto__: null,
-        setTimeout: () => {
+        '__proto__': null,
+        'scheduler.wait': () => {
+          this.#restoreOriginalSchedulerWait();
+        },
+        'setTimeout': () => {
           this.#restoreOriginalSetTimeout();
         },
-        setInterval: () => {
+        'setInterval': () => {
           this.#restoreOriginalSetInterval();
         },
-        setImmediate: () => {
+        'setImmediate': () => {
           this.#restoreSetImmediate();
         },
-        Date: () => {
+        'Date': () => {
           ObjectDefineProperty(globalThis, 'Date', this.#nativeDateDescriptor);
         },
       },
@@ -688,13 +715,9 @@ class MockTimers {
       throw new ERR_INVALID_ARG_VALUE('now', internalOptions.now, `epoch must be a positive integer received ${internalOptions.now}`);
     }
 
-    if (!internalOptions.now) {
-      internalOptions.now = 0;
-    }
+    internalOptions.now ||= 0;
 
-    if (!internalOptions.apis) {
-      internalOptions.apis = SUPPORTED_APIS;
-    }
+    internalOptions.apis ||= SUPPORTED_APIS;
 
     validateStringArray(internalOptions.apis, 'options.apis');
     // Check that the timers passed are supported
