@@ -19,9 +19,11 @@ const {
   PromisePrototypeThen,
   PromiseReject,
   PromiseResolve,
+  PromiseWithResolvers,
   SafePromiseAll,
   Symbol,
   SymbolAsyncIterator,
+  SymbolDispose,
   SymbolToStringTag,
   TypedArrayPrototypeGetLength,
   Uint8Array,
@@ -49,12 +51,10 @@ const {
 } = require('internal/util/types');
 
 const {
-  createDeferredPromise,
   customInspectSymbol: kInspect,
   kEmptyObject,
   kEnumerableProperty,
   SideEffectFreeRegExpPrototypeSymbolReplace,
-  SymbolDispose,
 } = require('internal/util');
 
 const {
@@ -74,6 +74,7 @@ const {
   kTransfer,
   kTransferList,
   markTransferMode,
+  structuredClone,
 } = require('internal/worker/js_transferable');
 
 const {
@@ -88,13 +89,12 @@ const {
   kControllerErrorFunction,
 } = require('internal/streams/utils');
 
-const { structuredClone } = internalBinding('messaging');
-
 const {
   ArrayBufferViewGetBuffer,
   ArrayBufferViewGetByteLength,
   ArrayBufferViewGetByteOffset,
   AsyncIterator,
+  canCopyArrayBuffer,
   cloneAsUint8Array,
   copyArrayBuffer,
   createPromiseCallback,
@@ -253,7 +253,7 @@ class ReadableStream {
     validateObject(strategy, 'strategy', kValidateObjectAllowObjectsAndNull);
     this[kState] = createReadableStreamState();
 
-    this[kIsClosedPromise] = createDeferredPromise();
+    this[kIsClosedPromise] = PromiseWithResolvers();
     this[kControllerErrorFunction] = () => {};
 
     // The spec requires handling of the strategy first
@@ -487,7 +487,7 @@ class ReadableStream {
           new ERR_INVALID_STATE.TypeError(
             'The reader is not bound to a ReadableStream'));
       }
-      const promise = createDeferredPromise();
+      const promise = PromiseWithResolvers();
 
       // eslint-disable-next-line no-use-before-define
       readableStreamDefaultReaderRead(reader, new ReadableStreamAsyncIteratorReadRequest(reader, state, promise));
@@ -639,11 +639,12 @@ ObjectDefineProperties(ReadableStream, {
 });
 
 function InternalTransferredReadableStream() {
+  ObjectSetPrototypeOf(this, ReadableStream.prototype);
   markTransferMode(this, false, true);
   this[kType] = 'ReadableStream';
   this[kState] = createReadableStreamState();
 
-  this[kIsClosedPromise] = createDeferredPromise();
+  this[kIsClosedPromise] = PromiseWithResolvers();
 }
 
 ObjectSetPrototypeOf(InternalTransferredReadableStream.prototype, ReadableStream.prototype);
@@ -786,7 +787,7 @@ class ReadableStreamAsyncIteratorReadRequest {
 
 class DefaultReadRequest {
   constructor() {
-    this[kState] = createDeferredPromise();
+    this[kState] = PromiseWithResolvers();
   }
 
   [kChunk](value) {
@@ -806,7 +807,7 @@ class DefaultReadRequest {
 
 class ReadIntoRequest {
   constructor() {
-    this[kState] = createDeferredPromise();
+    this[kState] = PromiseWithResolvers();
   }
 
   [kChunk](value) {
@@ -1226,10 +1227,11 @@ ObjectDefineProperties(ReadableByteStreamController.prototype, {
 });
 
 function InternalReadableStream(start, pull, cancel, highWaterMark, size) {
+  ObjectSetPrototypeOf(this, ReadableStream.prototype);
   markTransferMode(this, false, true);
   this[kType] = 'ReadableStream';
   this[kState] = createReadableStreamState();
-  this[kIsClosedPromise] = createDeferredPromise();
+  this[kIsClosedPromise] = PromiseWithResolvers();
   const controller = new ReadableStreamDefaultController(kSkipThrow);
   setupReadableStreamDefaultController(
     this,
@@ -1253,10 +1255,11 @@ function createReadableStream(start, pull, cancel, highWaterMark = 1, size = ()
 }
 
 function InternalReadableByteStream(start, pull, cancel) {
+  ObjectSetPrototypeOf(this, ReadableStream.prototype);
   markTransferMode(this, false, true);
   this[kType] = 'ReadableStream';
   this[kState] = createReadableStreamState();
-  this[kIsClosedPromise] = createDeferredPromise();
+  this[kIsClosedPromise] = PromiseWithResolvers();
   const controller = new ReadableByteStreamController(kSkipThrow);
   setupReadableByteStreamController(
     this,
@@ -1390,7 +1393,7 @@ function readableStreamPipeTo(
     }
   }
 
-  const promise = createDeferredPromise();
+  const promise = PromiseWithResolvers();
 
   const state = {
     currentWrite: PromiseResolve(),
@@ -1507,7 +1510,7 @@ function readableStreamPipeTo(
 
     await writer[kState].ready.promise;
 
-    const promise = createDeferredPromise();
+    const promise = PromiseWithResolvers();
     // eslint-disable-next-line no-use-before-define
     readableStreamDefaultReaderRead(reader, new PipeToReadableStreamReadRequest(writer, state, promise));
 
@@ -1611,7 +1614,7 @@ function readableStreamDefaultTee(stream, cloneForBranch2) {
   let reason2;
   let branch1;
   let branch2;
-  const cancelPromise = createDeferredPromise();
+  const cancelPromise = PromiseWithResolvers();
 
   async function pullAlgorithm() {
     if (reading) return;
@@ -1710,7 +1713,7 @@ function readableByteStreamTee(stream) {
   let reason2;
   let branch1;
   let branch2;
-  const cancelDeferred = createDeferredPromise();
+  const cancelDeferred = PromiseWithResolvers();
 
   function forwardReaderError(thisReader) {
     PromisePrototypeThen(
@@ -2157,7 +2160,7 @@ function readableStreamReaderGenericInitialize(reader, stream) {
   stream[kState].reader = reader;
   switch (stream[kState].state) {
     case 'readable':
-      reader[kState].close = createDeferredPromise();
+      reader[kState].close = PromiseWithResolvers();
       break;
     case 'closed':
       reader[kState].close = {
@@ -2550,6 +2553,15 @@ function readableByteStreamControllerCommitPullIntoDescriptor(stream, desc) {
   }
 }
 
+function readableByteStreamControllerCommitPullIntoDescriptors(stream, descriptors) {
+  for (let i = 0; i < descriptors.length; ++i) {
+    readableByteStreamControllerCommitPullIntoDescriptor(
+      stream,
+      descriptors[i],
+    );
+  }
+}
+
 function readableByteStreamControllerInvalidateBYOBRequest(controller) {
   if (controller[kState].byobRequest === null)
     return;
@@ -2756,11 +2768,11 @@ function readableByteStreamControllerRespondInClosedState(controller, desc) {
     stream,
   } = controller[kState];
   if (readableStreamHasBYOBReader(stream)) {
-    while (readableStreamGetNumReadIntoRequests(stream) > 0) {
-      readableByteStreamControllerCommitPullIntoDescriptor(
-        stream,
-        readableByteStreamControllerShiftPendingPullInto(controller));
+    const filledPullIntos = [];
+    for (let i = 0; i < readableStreamGetNumReadIntoRequests(stream); ++i) {
+      ArrayPrototypePush(filledPullIntos, readableByteStreamControllerShiftPendingPullInto(controller));
     }
+    readableByteStreamControllerCommitPullIntoDescriptors(stream, filledPullIntos);
   }
 }
 
@@ -2841,8 +2853,9 @@ function readableByteStreamControllerEnqueue(controller, chunk) {
       transferredBuffer,
       byteOffset,
       byteLength);
-    readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(
+    const filledPullIntos = readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(
       controller);
+    readableByteStreamControllerCommitPullIntoDescriptors(stream, filledPullIntos);
   } else {
     assert(!isReadableStreamLocked(stream));
     readableByteStreamControllerEnqueueChunkToQueue(
@@ -2935,6 +2948,7 @@ function readableByteStreamControllerFillPullIntoDescriptorFromQueue(
   const maxAlignedBytes = maxBytesFilled - (maxBytesFilled % elementSize);
   let totalBytesToCopyRemaining = maxBytesToCopy;
   let ready = false;
+  assert(!ArrayBufferPrototypeGetDetached(buffer));
   assert(bytesFilled < minimumFill);
   if (maxAlignedBytes >= minimumFill) {
     totalBytesToCopyRemaining = maxAlignedBytes - bytesFilled;
@@ -2950,12 +2964,12 @@ function readableByteStreamControllerFillPullIntoDescriptorFromQueue(
       totalBytesToCopyRemaining,
       headOfQueue.byteLength);
     const destStart = byteOffset + desc.bytesFilled;
-    const arrayBufferByteLength = ArrayBufferPrototypeGetByteLength(buffer);
-    if (arrayBufferByteLength - destStart < bytesToCopy) {
-      throw new ERR_INVALID_STATE.RangeError(
-        'view ArrayBuffer size is invalid');
-    }
-    assert(arrayBufferByteLength - destStart >= bytesToCopy);
+    assert(canCopyArrayBuffer(
+      buffer,
+      destStart,
+      headOfQueue.buffer,
+      headOfQueue.byteOffset,
+      bytesToCopy));
     copyArrayBuffer(
       buffer,
       destStart,
@@ -2989,26 +3003,30 @@ function readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(
   const {
     closeRequested,
     pendingPullIntos,
-    stream,
   } = controller[kState];
   assert(!closeRequested);
+  const filledPullIntos = [];
   while (pendingPullIntos.length) {
     if (!controller[kState].queueTotalSize)
-      return;
+      break;
     const desc = pendingPullIntos[0];
     if (readableByteStreamControllerFillPullIntoDescriptorFromQueue(
       controller,
       desc)) {
       readableByteStreamControllerShiftPendingPullInto(controller);
-      readableByteStreamControllerCommitPullIntoDescriptor(stream, desc);
+      ArrayPrototypePush(filledPullIntos, desc);
     }
   }
+  return filledPullIntos;
 }
 
 function readableByteStreamControllerRespondInReadableState(
   controller,
   bytesWritten,
   desc) {
+  const {
+    stream,
+  } = controller[kState];
   const {
     buffer,
     bytesFilled,
@@ -3029,9 +3047,10 @@ function readableByteStreamControllerRespondInReadableState(
       controller,
       desc,
     );
-    readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(
+    const filledPullIntos = readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(
       controller,
     );
+    readableByteStreamControllerCommitPullIntoDescriptors(stream, filledPullIntos);
     return;
   }
 
@@ -3057,10 +3076,10 @@ function readableByteStreamControllerRespondInReadableState(
       ArrayBufferPrototypeGetByteLength(remainder));
   }
   desc.bytesFilled -= remainderSize;
-  readableByteStreamControllerCommitPullIntoDescriptor(
-    controller[kState].stream,
-    desc);
-  readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(controller);
+  const filledPullIntos = readableByteStreamControllerProcessPullIntoDescriptorsUsingQueue(controller);
+
+  readableByteStreamControllerCommitPullIntoDescriptor(stream, desc);
+  readableByteStreamControllerCommitPullIntoDescriptors(stream, filledPullIntos);
 }
 
 function readableByteStreamControllerRespondWithNewView(controller, view) {
